package es.ucm.fdi.grasia.faerie.observer.internal.remote.r_osgi;

import static es.ucm.fdi.grasia.faerie.core.util.ContextURIs.localPort;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Arrays;

import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Reference;
import org.apache.felix.scr.annotations.Service;
import org.apache.log4j.Logger;

import ch.ethz.iks.r_osgi.RemoteOSGiService;
import ch.ethz.iks.r_osgi.RemoteServiceReference;
import ch.ethz.iks.r_osgi.service_discovery.ServiceDiscoveryListener;
import es.ucm.fdi.grasia.faerie.core.ContextObserver;

/**
 * 
 * @author Chema
 */
@Service
@Component(immediate = true)
public class RemoteOSGiListener implements ServiceDiscoveryListener {

	@Reference
	private RemoteOSGiService remoteOSGiService = null;

	public void bindRemoteOSGiService(RemoteOSGiService remoteOSGiService) {
		Logger.getLogger(this.getClass()).trace("(" + remoteOSGiService + ") * " + this);
		this.remoteOSGiService = remoteOSGiService;
	}

	public void unbindRemoteOSGiService(RemoteOSGiService remoteOSGiService) {
		Logger.getLogger(this.getClass()).trace("(" + remoteOSGiService + ") * " + this);
		this.remoteOSGiService = null;
	}

	private String propertiesToString(RemoteServiceReference ref) {
		StringBuilder props = new StringBuilder();
		props.append(ref);
		String[] keys = ref.getPropertyKeys();
		for (int i = 0; i < keys.length; ++i) {
			props.append("\n\t").append(keys[i]).append(" = ");
			Object obj = ref.getProperty(keys[i]);
			if (obj.getClass().isArray())
				props.append(Arrays.toString((Object[]) obj));
			else
				props.append(ref.getProperty(keys[i]));
		}
		return props.toString();
	}

	@Override
	public void announceService(String name, ch.ethz.iks.r_osgi.URI uri) {
		try {
			Logger.getLogger(this.getClass()).debug("Announcing " + name + " on " + uri);
			if (name.equals(ContextObserver.class.getName())) {
				InetAddress localHost = InetAddress.getLocalHost();
				InetAddress remoteHost = InetAddress.getByName(uri.getHost());
				if (!remoteHost.equals(localHost) || uri.getPort() != localPort()) {
					RemoteServiceReference ref = remoteOSGiService.getRemoteServiceReference(uri);
					Logger.getLogger(this.getClass()).debug(propertiesToString(ref));
					remoteOSGiService.getRemoteService(ref);
				}
			}
		} catch (UnknownHostException ex) {
			Logger.getLogger(this.getClass()).warn(ex.getLocalizedMessage(), ex);
		}
	}

	@Override
	public void discardService(String name, ch.ethz.iks.r_osgi.URI uri) {
		try {
			Logger.getLogger(this.getClass()).debug("Discarding " + name + " on " + uri);
			if (name.equals(ContextObserver.class.getName())) {
				InetAddress localHost = InetAddress.getLocalHost();
				InetAddress remoteHost = InetAddress.getByName(uri.getHost());
				if (!remoteHost.equals(localHost) || uri.getPort() != localPort()) {
					RemoteServiceReference ref = remoteOSGiService.getRemoteServiceReference(uri);
					remoteOSGiService.ungetRemoteService(ref);
				}
			}
		} catch (UnknownHostException ex) {
			Logger.getLogger(this.getClass()).warn(ex.getLocalizedMessage(), ex);
		}
	}
}
